import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches


def spider_plot(title, radial_labels, case_data, case_labels, first_color=0):
    """
    Minimum example of the inputs
    :param title: 'Titulo'
    :param radial_labels: ["medida 1", "medida 2", "medida 3"]
    :param case_data: [[0.1, 0.2, 0.5],
                       [0.3, 0.4, 0.6]]
    :param case_labels: ["Serie 1", "Serie 2"]
    :param first_color: 0 (default)
    :return:
    """
    n_radial_labels = len(radial_labels)

    # What will be the angle of each axis in the plot?
    angles = [np.pi * (0.5 + (1 + 2 * n) / float(n_radial_labels)) for n in range(n_radial_labels)]
    for i, a in enumerate(angles):
        if a > np.pi * 2:
            angles[i] -= np.pi * 2
    angles += angles[:1]

    # Initialise the spider plot
    fig = plt.figure(figsize=(9, 7))
    ax = plt.subplot(111, polar=True, frame_on=False)

    # Draw one axe per variable + add labels labels yet
    plt.xticks(angles[:-1], radial_labels, size='large')

    ax.plot(angles, [1.0] * len(angles), linewidth=1, linestyle='solid', color='black')

    # grey half circle
    half_axis = np.concatenate(([angles[0] - np.pi / 14], angles[:8], [angles[7] + np.pi / 14] * 2))
    half_circle = [0.98] + [1] * 8 + [0.98] + [0]
    ax.fill(half_axis, half_circle, facecolor="grey", alpha=0.25)  # Fill the polygon

    #    axes.set_title(title, weight='bold', position=(0.5, 1.1),
    #                   horizontalalignment='center', verticalalignment='center')

    if len(case_labels) > 10:
        print("Be careful! I cannot plot more than 10 labels.")
    colors = ["C%d" % (i + first_color) for i in range(len(case_labels))]
    # colors = colors[first_color:] + colors[:first_color]
    ax.set_rgrids([0.2, 0.4, 0.6, 0.8], [0.8, 0.6, 0.4, 0.2])
    ax.set_ylim([0, 1])

    # ------- PART 2: Add plots

    # Plot each individual = each line of the data
    for i, (cl, color) in enumerate(zip(case_labels, colors)):
        values = case_data[i]  # df.loc[1].drop('group').values.flatten().tolist()
        values += values[:1]
        ax.plot(angles, values, linewidth=2, linestyle='solid', label=cl, color=color)

    # Block- and frame-based legend
    frame_patch = mpatches.Patch(facecolor='lightgrey', edgecolor='k', label='Frame-based metrics')
    block_patch = mpatches.Patch(facecolor='white', edgecolor='k', label='Block-based metrics')
    first_legend = plt.legend(handles=[frame_patch, block_patch], ncol=1,
                              bbox_to_anchor=(-0.15, 0.0), loc=2, borderaxespad=0.0,
                              framealpha=0, fontsize='large')
    plt.gca().add_artist(first_legend)

    # Add legend
    plt.legend(loc='upper left', bbox_to_anchor=(1.15, 1.05),
               labelspacing=0.5, fontsize='large')

    plt.tight_layout(pad=3.5)
    plt.savefig("spider_" + title + ".pdf")
    plt.savefig("spider_" + title + ".png")
    plt.show()


def print_f1scores_from_report(single_activity_report):
    print("Predictor".ljust(25) + "Frame-based f1score".ljust(25) + "Block-based f1score")

    for predictor_name, predictor_report in single_activity_report.groupby("predictor_name", sort=False):
        frame_str = "%0.3f (+-%0.3f)" % (predictor_report.frame_f1score.mean(),
                                         predictor_report.frame_f1score.std())
        event_str = "%0.3f (+-%0.3f)" % (predictor_report.event_f1score.mean(),
                                         predictor_report.event_f1score.std())
        print(predictor_name.ljust(25)[:25] + frame_str.ljust(25) + event_str)


def plot_spider_from_report(single_activity_report, first_color):
    activity = single_activity_report.activity.iloc[0]
    case_data = []
    case_labels = []

    for predictor_name, predictor_report in single_activity_report.groupby("predictor_name", sort=False):
        predictor_mean = predictor_report.mean()
        # Saving data to plot
        case_data.append([predictor_mean.frame_recall, predictor_mean.frame_precision,
                          1 - predictor_mean.f_rate, 1 - predictor_mean.m_rate,
                          1 - predictor_mean.d_rate, 1 - predictor_mean.i_rate,
                          1 - predictor_mean.u_rate, 1 - predictor_mean.o_rate,
                          #
                          1 - predictor_mean.ins_rate, 1 - predictor_mean.del_rate,
                          1 - predictor_mean.merge_rate, 1 - predictor_mean.frag_rate,
                          predictor_mean.event_precision, predictor_mean.event_recall
                          ])
        case_labels.append(predictor_name)

    spider_plot(title=activity,
                # Labels are organized
                radial_labels=[  # Labels are given in anti-clockwise direction
                    # Frame-based metrics
                    # "FNR", "FDR", "Frag.", "Merg.", "Del.", "Ins.",
                    r"$\rm{FNR}_f$", r"$\rm{FDR}_f$", r"$\rm{F}_f$", r"$\rm{M}_f$", r"$\rm{D}_f$",
                    r"$\rm{I}_f$",
                    # "Under.", "Over.",
                    r"$\rm{U}_f$", r"$\rm{O}_f$",
                    # Block-based metrics
                    # "Ins.", "Del.", "Merg.", "Frag.", "FDR", "FNR"
                    r"$\rm{I}_b$", r"$\rm{D}_b$", r"$\rm{M}_b$", r"$\rm{F}_b$", r"$\rm{FDR}_b$", r"$\rm{FNR}_b$"
                ],
                case_data=case_data,
                case_labels=case_labels,
                first_color=first_color)


def plot_violinplot_from_report(single_activity_report, first_color=0):
    grouped_reports = single_activity_report.groupby("predictor_name", sort=False)
    n_predictors = len(grouped_reports)
    predictors_labels = []
    ground_mean_in_minutes = single_activity_report.ground_positives.mean() / 60.0
    activity = single_activity_report.activity.iloc[0]

    if float(ground_mean_in_minutes) == 0.0:
        print("\nWARNING: Label %s is present in none of your ground files\n" % activity)
        return None
    print("Mean duration used to normalize the violinplot top axis: %.2f min\n" % ground_mean_in_minutes)

    plt.figure()
    pos = np.arange(n_predictors) + .5  # the bar centers on the y axis

    if n_predictors > 10:
        print("Be careful! I cannot plot more than 10 labels.")
    colors = ["C%d" % (i + first_color) for i in range(n_predictors)]
    # colors = colors[first_color:] + colors[:first_color]

    for (predictor_name, predictor_report), p, c in zip(grouped_reports, pos, colors):
        predictors_labels.append(predictor_name)

        error_in_minutes = predictor_report.raw_time_error.values / 60.0
        parts = plt.violinplot(error_in_minutes[np.isfinite(error_in_minutes)], [p], points=50, vert=False, widths=0.65,
                               showmeans=False, showmedians=True, showextrema=True, bw_method='silverman')
        for pc in parts['bodies']:
            pc.set_facecolor(c)
        for part_name in ["cmaxes", "cmins", "cbars", "cmedians"]:
            parts[part_name].set_color(c)

    plt.axvline(x=0, color="k", linestyle="dashed")
    plt.yticks(pos, predictors_labels)
    plt.gca().invert_yaxis()
    plt.minorticks_on()
    plt.xlabel('Time Estimation Error (min)')

    ax1 = plt.gca()
    ax2 = ax1.twiny()
    ax2.set_xlim(ax1.get_xlim())
    ax2.set_xticklabels(["%.0f%%" % (100.0 * d / ground_mean_in_minutes) for d in ax1.get_xticks()])
    ax2.set_xlabel("Normalized Error")
    plt.minorticks_on()

    plt.tight_layout()
    plt.savefig('violin_' + activity + '.pdf')
    plt.savefig('violin_' + activity + '.png')
    plt.show()


def _format_time_errors(report_row):
    error_in_minutes = report_row.raw_time_error / 60.0
    print("\t{}\t{:0.2f} min, \t{:0.3f}".format(report_row.ground_filename, error_in_minutes, report_row.frame_f1score))


def print_time_errors_from_report(single_activity_report):
    grouped_reports = single_activity_report.groupby("predictor_name", sort=False)
    for predictor_name, predictor_report in grouped_reports:
        print("\n>>{} errors (in minutes), frame f1-score\n".format(predictor_name))
        predictor_report.apply(_format_time_errors, axis="columns")



import numpy as np
from armetrics.models import Event, Segment


def segment_basic_score(y_true_seg, y_pred_seg):
    """
    Compares y_true_seg with y_pred_seg and returns the corresponding label

    :param y_true_seg: true value of segment
    :param y_pred_seg: predicted value of segment
    :return: label that indicates True Positive, True Negative, False Positive or False Negative.
    Possible outcomes: "TP", "TN", "FP", or "FN".
    """
    true_vs_pred = {(True, True): "TP",
                    (True, False): "FN",
                    (False, True): "FP",
                    (False, False): "TN"}

    return true_vs_pred[(y_true_seg, y_pred_seg)]


def score_segments(basic_scored_segments):
    """
    Transform basic labels "TP", "TN", "FN", "FP" to:
    Correct (C)
    Correct Null ("")
    Insertion (I)
    Merge (M)
    Overfill (O). starting (Oa), ending (Oz)
    Deletion (D)
    Fragmenting (F)
    Underfill (U), starting (Ua), ending (Uz)

    :param basic_scored_segments:
    List of basic scores assigned to segments
    :return:
    List of advanced scores assigned to segments
    """

    segments = [Segment(seg.start, seg.end) for seg in basic_scored_segments]

    # First segment relabel
    aux = ''.join(seg.label for seg in basic_scored_segments[:2])
    if basic_scored_segments[0].label in ["TP"]:
        segments[0].label = "C"  # Correct
    elif basic_scored_segments[0].label in ["TN"]:
        segments[0].label = ""  # Correct null
    elif aux in ["FPTN", "FPFN"]:
        segments[0].label = "I"  # Insertion
    elif aux in ["FNTN", "FNFP"]:
        segments[0].label = "D"  # Deletion
    elif aux in ["FPTP"]:
        segments[0].label = "Oa"  # starting Overfill
    elif aux in ["FNTP"]:
        segments[0].label = "Ua"  # starting Underfill

    # Middle segment relabel
    for i in range(1, len(basic_scored_segments) - 1):
        aux = ''.join(seg.label for seg in basic_scored_segments[i - 1:i + 2])
        if basic_scored_segments[i].label in ["TP"]:
            segments[i].label = "C"  # Correct
        elif basic_scored_segments[i].label in ["TN"]:
            segments[i].label = ""  # Correct null
        elif aux in ["TPFPTP"]:
            segments[i].label = "M"  # Merge
        elif aux in ["TPFNTP"]:
            segments[i].label = "F"  # Fragmentation
        elif aux in ["TNFPTN", "FNFPTN", "TNFPFN", "FNFPFN"]:
            segments[i].label = "I"  # Insertion
        elif aux in ["TNFNTN", "FPFNTN", "TNFNFP", "FPFNFP"]:
            segments[i].label = "D"  # Deletion
        elif aux in ["TNFPTP", "FNFPTP"]:
            segments[i].label = "Oa"  # starting Overfill
        elif aux in ["TPFPTN", "TPFPFN"]:
            segments[i].label = "Oz"  # ending Overfill
        elif aux in ["TNFNTP", "FPFNTP"]:
            segments[i].label = "Ua"  # starting Underfill
        elif aux in ["TPFNTN", "TPFNFP"]:
            segments[i].label = "Uz"  # ending Underfill

    if len(basic_scored_segments) > 1:
        # Last segment relabel
        aux = ''.join(seg.label for seg in basic_scored_segments[-2:])
        if basic_scored_segments[-1].label in ["TP"]:
            segments[-1].label = "C"  # Correct
        elif basic_scored_segments[-1].label in ["TN"]:
            segments[-1].label = ""  # Correct null
        elif aux in ["TNFP", "FNFP"]:
            segments[-1].label = "I"  # Insertion
        elif aux in ["TNFN", "FPFN"]:
            segments[-1].label = "D"  # Deletion
        elif aux in ["TPFP"]:
            segments[-1].label = "Oa"  # ending Overfill
        elif aux in ["TPFN"]:
            segments[-1].label = "Ua"  # ending Underfill

    return segments


def score_events(scored_segments, true_events, pred_events):
    # Create list of true and predicted events with empty labels
    scored_true_evs = [Event(start, end) for start, end in true_events]
    scored_pred_evs = [Event(start, end) for start, end in pred_events]

    # True events labeling, first pass (using labeled_segments)
    for true_ev in scored_true_evs:
        for seg in scored_segments:
            if true_ev.start <= seg.start <= true_ev.end:
                # In the first pass, D and F segments are assigned to true events
                if seg.label in ["D", "F"]:
                    true_ev.add_label(seg.label)

    # Pred events labeling, first pass (using labeled_segments)
    for pred_ev in scored_pred_evs:
        for seg in scored_segments:
            if pred_ev.start <= seg.start <= pred_ev.end:
                # In the first pass, I and M segments are assigned to pred events
                if seg.label in ["I", "M"]:
                    pred_ev.add_label(seg.label)

    # True events labeling, second pass (using labels of prediction)
    for true_ev in scored_true_evs:
        for pred_ev in scored_pred_evs:
            if pred_ev.overlap(true_ev):
                if pred_ev.label in ["M", "FM"]:
                    true_ev.add_label("M")

    # Pred events labeling, second pass (using labels of ground truth)
    for pred_ev in scored_pred_evs:
        for true_ev in scored_true_evs:
            if true_ev.overlap(pred_ev):
                if true_ev.label in ["F", "FM"]:
                    pred_ev.add_label("F")

    # If no label was assigned so far, then it is a correct detected event
    for true_ev in scored_true_evs:
        if true_ev.label == "":
            true_ev.add_label("C")

    for pred_ev in scored_pred_evs:
        if pred_ev.label == "":
            pred_ev.add_label("C")

    return scored_true_evs, scored_pred_evs


def events_summary(scored_true_events, scored_pred_events, normalize=True):
    scored_true_events = [e.label for e in scored_true_events]
    scored_pred_events = [e.label for e in scored_pred_events]

    summary = {"C": scored_true_events.count("C"),     # Total correct events
               # ground truth
               "D": scored_true_events.count("D"),     # Total deleted events
               "F": scored_true_events.count("F"),     # Total fragmented events
               "FM": scored_true_events.count("FM"),   # Total fragmented and merged events
               "M": scored_true_events.count("M"),     # Total merged events
               # predicted output
               "C'": scored_pred_events.count("C"),    # Total correct events (equivalent to C if not normalized)
               "I'": scored_pred_events.count("I"),    # Total inserted events
               "F'": scored_pred_events.count("F"),    # Total fragmenting events
               "FM'": scored_pred_events.count("FM"),  # Total fragmenting and merging events
               "M'": scored_pred_events.count("M"),     # Total merging events
               "num_true_events": len(scored_true_events),
               "num_pred_events": len(scored_pred_events)
               }

    if normalize:
        # Normalized true events metrics
        for lab in ["C", "D", "F", "FM", "M"]:
            if summary["num_true_events"] > 0:
                summary[lab+"_rate"] = summary[lab] / max(1, summary["num_true_events"])
            else:
                summary[lab + "_rate"] = np.nan
        # Normalized predicted events metrics
        for lab in ["C'", "I'", "F'", "FM'", "M'"]:
            if summary["num_pred_events"] > 0:
                summary[lab+"_rate"] = summary[lab] / max(1, summary["num_pred_events"])
            else:
                summary[lab + "_rate"] = np.nan

    if summary["num_true_events"] > 0:
        summary["event_recall"] = 1.0 * summary["C"] / summary["num_true_events"]

        summary["frag_rate"] = 1.0 * sum(summary[l] for l in ["F", "FM"]) / summary["num_true_events"]
        summary["merge_rate"] = 1.0 * sum(summary[l] for l in ["M", "FM"]) / summary["num_true_events"]
        summary["del_rate"] = 1.0 * sum(summary[l] for l in ["D"]) / summary["num_true_events"]
    else:
        summary["event_recall"] = np.nan

        summary["frag_rate"] = np.nan
        summary["merge_rate"] = np.nan
        summary["del_rate"] = np.nan

    if summary["num_pred_events"] > 0:
        summary["event_precision"] = 1.0 * summary["C"] / summary["num_pred_events"]

        summary["ins_rate"] = 1.0 * sum(summary[l] for l in ["I'"]) / summary["num_pred_events"]
    else:
        summary["event_precision"] = np.nan
        summary["ins_rate"] = np.nan

    if summary["event_recall"] > 0 and summary["event_precision"] > 0:
        summary["event_f1score"] = 2 * summary["event_recall"] * summary["event_precision"] / (
            summary["event_recall"] + summary["event_precision"])
    else:
        summary["event_f1score"] = np.nan

    return summary


def frames_summary(scored_frames, normalize=True):
    # TODO add docstring

    summary = {"tp": scored_frames.count("C"),    # Total correct frames (true positives)
               "tn": scored_frames.count(""),     # Total correct frames (true negatives)
               "d": scored_frames.count("D"),     # Total deleted frames
               "f": scored_frames.count("F"),     # Total fragmented frames
               "i": scored_frames.count("I"),     # Total inserted frames
               "m": scored_frames.count("M"),     # Total merged frames
               "ua": scored_frames.count("Ua"),   # Total starting underfill frames
               "uz": scored_frames.count("Uz"),   # Total ending underfill frames
               "oa": scored_frames.count("Oa"),   # Total starting overfill frames
               "oz": scored_frames.count("Oz"),   # Total ending overfill frames
               }

    summary["u"] = summary["ua"] + summary["uz"]  # Total underfill frames
    summary["o"] = summary["oa"] + summary["oz"]  # Total overfill frames
    summary["ground_positives"] = sum(summary[lab] for lab in ["tp", "f", "d", "ua", "uz"])
    summary["ground_negatives"] = sum(summary[lab] for lab in ["tn", "m", "i", "oa", "oz"])
    summary["output_positives"] = sum(summary[lab] for lab in ["tp", "m", "i", "oa", "oz"])
    summary["output_negatives"] = sum(summary[lab] for lab in ["tn", "f", "d", "ua", "uz"])

    summary["fp"] = summary["output_positives"] - summary["tp"]
    summary["fn"] = summary["output_negatives"] - summary["tn"]
    if (summary["tp"] + summary["fp"]) > 0:
        summary["frame_precision"] = 1.0 * summary["tp"] / (summary["tp"] + summary["fp"])
    else:
        summary["frame_precision"] = np.nan
    if (summary["tp"] + summary["fn"]) > 0:
        summary["frame_recall"] = 1.0 * summary["tp"] / (summary["tp"] + summary["fn"])
    else:
        summary["frame_recall"] = np.nan

    if summary["frame_recall"] > 0 and summary["frame_precision"] > 0:
        summary["frame_f1score"] = 2 * summary["frame_recall"] * summary["frame_precision"] / (
            summary["frame_recall"] + summary["frame_precision"])
    else:
        summary["frame_f1score"] = np.nan

    if normalize:
        # Normalized positives frame metrics
        for lab in ["tp", "d", "f", "ua", "uz", "u", "fp"]:
            if summary["ground_positives"] > 0:
                summary[lab+"_rate"] = summary[lab] / max(1, summary["ground_positives"])
            else:
                summary[lab + "_rate"] = np.nan
        # Normalized predicted events metrics
        for lab in ["tn", "i", "m", "oa", "oz", "o", "fn"]:
            if summary["ground_negatives"] > 0:
                summary[lab+"_rate"] = summary[lab] / max(1, summary["ground_negatives"])
            else:
                summary[lab + "_rate"] = np.nan

    # FIXME Chequear que este funcionando bien cuando no hay etiquetas positivas en la referencia
    summary["raw_time_error"] = summary["output_positives"] - summary["ground_positives"]
    # if summary["tp"] + summary["fn"] > 0:
    summary["matching_time"] = summary["output_positives"] / max(1, summary["tp"] + summary["fn"])  # Matching time
    # else:
    #     summary["matching_time"] = np.nan

    return summary




class Event:
    def __init__(self, start=None, end=None, label=""):
        self.start = start
        self.end = end
        self.label = label

    def __repr__(self):
        return str([self.start, self.end, self.label])

    def add_label(self, label):
        if label not in self.label:
            self.label += label
            if self.label == "MF":
                self.label = "FM"

    def overlap(self, event):
        """
        Return True if given event overlaps with this one (at the beginning or the end).
        :param event:
        :return:
        """
        return self.start <= event.start < self.end or self.start < event.end <= self.end

    def __len__(self):
        return self.end - self.start


class Segment:
    def __init__(self, start=None, end=None, label=""):
        self.start = start
        self.end = end
        self.label = label

    def __repr__(self):
        return str([self.start, self.end, self.label])

    def __len__(self):
        return self.end - self.start



import numpy as np
import pandas as pd
import os

from armetrics.models import Segment
from armetrics import scorer
from armetrics import plotter


def frames2segments(y_true, y_pred, advanced_labels=True):
    """
    Compute segment boundaries and compare y_true with y_pred.

    Segments are derived by comparing y_true with y_pred:
    any change in either y_pred or y_true marks a segment boundary.
    First-segment start-index is 0 and last-segment end-index is len(y_true).

    :param y_true: array_like
        ground truth

    :param y_pred: array_like
        prediction or classifier output

    :param advanced_labels: (Default True)
        Defines what kind of labels to return

    :return: tuple (3 columns),
    (array_like) first column corresponds to starts,
    (array_like) second column corresponds to ends,
    (list) third column corresponds to basic labels (TP, TN, FP, FN)
    or advanced labels (C, I, D, M, F, Oa, Oz, Ua, Uz)
    """
    # Pad with zeros
    max_len = max(len(y_true), len(y_pred))
    y_true = np.pad(y_true, (0, max_len - len(y_true)), "constant")
    y_pred = np.pad(y_pred, (0, max_len - len(y_pred)), "constant")

    y_true_breaks = np.flatnonzero(np.diff(y_true)) + 1  # locate changes in y_true
    y_pred_breaks = np.flatnonzero(np.diff(y_pred)) + 1  # locate changes in y_pred
    seg_breaks = np.union1d(y_true_breaks, y_pred_breaks)  # define segment breaks
    seg_starts = np.append([0], seg_breaks)  # add 0 as the first start
    seg_ends = np.append(seg_breaks, [len(y_true)])  # append len(y_true) as the last end

    # Compare segments at their first element to get corresponding labels
    seg_basic_labels = [scorer.segment_basic_score(y_true[i], y_pred[i]) for i in seg_starts]
    segments = [Segment(start, end, label) for start, end, label in zip(seg_starts, seg_ends, seg_basic_labels)]
    if advanced_labels:
        segments = scorer.score_segments(segments)

    return segments


def binarize_frames(labeled_frames, class_to_keep):
    binarized = []
    for frame in labeled_frames:
        if frame == class_to_keep:
            binarized.append(1)
        else:
            binarized.append(0)
    return np.array(binarized, dtype=np.int8)


def binframes2events(bin_frames):
    breaks = np.flatnonzero(np.diff(bin_frames)) + 1  # locate changes in bin_frames
    starts = np.append([0], breaks)  # add 0 as the first start
    ends = np.append(breaks, [len(bin_frames)])  # append len(bin_frame) as the last end
    # Events are specified with one
    events = [[f_start, f_end] for f_start, f_end in zip(starts, ends) if bin_frames[f_start]]
    return events


def segments2frames(scored_segments):
    output = []
    for seg in scored_segments:
        output += [seg.label] * (seg.end - seg.start)
    return output


def events2frames(event_list, length=None):
    """
    Translate an event list into an array of binary frames.

    Event list comprising start and end indexes of events (must be positive).
     For example: [[3, 5], [8, 10]]

    Returns an np.array corresponding to frames.
     Frames that correspond to an event ar marked with 1.
     Frames that not correspond to an event ar marked with 0.

    :param length: (None by default)
     Extend the frame array to given length
    :param event_list:
    :return: frames:
    """
    frames = []
    for start_e, end_e in event_list:
        frames += [0] * (start_e - len(frames))
        frames += [1] * (end_e - start_e)
    if length:
        frames += [0] * (length - len(frames))
    return np.array(frames, dtype=np.int8)


def get_scores(y_true_bin, y_pred_bin):
    y_true_evs = binframes2events(y_true_bin)
    y_pred_evs = binframes2events(y_pred_bin)
    scored_segments = frames2segments(y_true_bin, y_pred_bin)
    scored_frames = segments2frames(scored_segments)
    scored_true_events, scored_pred_events = scorer.score_events(scored_segments, y_true_evs, y_pred_evs)

    return {"scored_true_events": scored_true_events,
            "scored_pred_events": scored_pred_events,
            "events_summary": scorer.events_summary(scored_true_events, scored_pred_events),
            "frames_summary": scorer.frames_summary(scored_frames)}


def get_sessions_scores(ground_filenames, prediction_filenames, loader_function, activities_of_interest,
                        starts_ends=None, **kwarg):
    """
    :param ground_filenames: list of filenames for the ground truth
    :param prediction_filenames: list of filenames for the predictor
    :param activities_of_interest: list of labels of interest (among the ones within given files)
    :param loader_function: function to read files, it should return a pandas Series
    (see load_txt_labels() for an example of load)
    :param starts_ends: (optional) list of tuples (start, end) in seconds for each ground file.
    It should has same length as ground_filenames
    Additions arguments are given to loader_function
    """
    if starts_ends is None:
        starts_ends = [(None, None)] * len(ground_filenames)

    # Ground sessions labels
    yground_by_session = [loader_function(filename, start=s, end=e, **kwarg)
                          for filename, (s, e) in zip(ground_filenames, starts_ends)]
    # Prediction sessions labels
    ypred_by_session = [loader_function(filename, start=s, end=e, **kwarg)
                        for filename, (s, e) in zip(prediction_filenames, starts_ends)]

    dfs_to_concat = []

    for ground_filename, pred_filename, ytest, ypred in zip(ground_filenames, prediction_filenames,
                                                            yground_by_session, ypred_by_session):
        for activity in activities_of_interest:
            ytest_bin = binarize_frames(ytest, activity)
            ypred_bin = binarize_frames(ypred, activity)
            scores_dic = get_scores(ytest_bin, ypred_bin)

            temp_merged = pd.DataFrame({**scores_dic["events_summary"],
                                        **scores_dic["frames_summary"],
                                        "activity":  activity,
                                        "ground_filename": os.path.basename(ground_filename),
                                        "prediction_filename": os.path.basename(pred_filename)
                                        }, index=[99])

            dfs_to_concat.append(temp_merged)

    df = pd.concat(dfs_to_concat, ignore_index=True)

    return df


def complete_report(csv_report_filename, labels_of_interest, labels_of_predictors, loader_function,
                    ground_filenames, *argv_prediction_filenames, display=True, first_color=0, **kwargs):
    """
    :param csv_report_filename: file to store results. If it is a an empty string it will save no file.
    :param labels_of_predictors: names of predictors to assign to predictions
    :param loader_function: function to load
    :param labels_of_interest: list of labels of interest (among the ones within given files)
    :param ground_filenames: list of filenames for the ground truth
    :param argv_prediction_filenames: lists of filenames for each method of prediction
    (each list is given as a separated argument)
    :param first_color: 0 (default) will be used to select the first color on plots
    :param starts_ends: (optional) list of tuples (start, end) in seconds for each ground file.
    It should has same length as ground_filenames
    :param display: True (default) plot results
    :type kwargs: extra arguments for loader function
    """
    # Get scores for each pair of ground and prediction files, for each activities of interest
    scored_sessions = [get_sessions_scores(ground_filenames, prediction_filenames,
                                           loader_function, labels_of_interest, **kwargs)
                       for prediction_filenames in argv_prediction_filenames]

    # Add name of predictors to scored sessions
    for lab, ss in zip(labels_of_predictors, scored_sessions):
        ss.insert(len(ss.columns), "predictor_name", lab)

    complete_report_df = pd.concat(scored_sessions, ignore_index=True)
    if csv_report_filename:
        complete_report_df.to_csv(csv_report_filename, index=False)

    if display:
        display_report(complete_report_df, first_color)

    return complete_report_df


def display_report(complete_report_df, first_color):
    report_activity_grouped = complete_report_df.groupby("activity", sort=False)

    for activity_label, single_activity_report in report_activity_grouped:
        print("\n================", activity_label, "================\n")

        plotter.plot_spider_from_report(single_activity_report, first_color)
        plotter.plot_violinplot_from_report(single_activity_report, first_color)
        plotter.print_f1scores_from_report(single_activity_report)
        plotter.print_time_errors_from_report(single_activity_report)


def load_txt_labels(filename, start=None, end=None, verbose=True, activities_of_interest=None):
    """
    Load activity labels given in filename and return a pandas Series compatible with the scorer.


    :param filename: path to the filename
    :param start: in seconds
    :param end:
    :param verbose:
    :param activities_of_interest:
    :return:
    """
    df = pd.read_table(filename, decimal=',', header=None)
    df.dropna(axis=1, how='all', inplace=True)
    df.columns = ["start", "end", "label"]

    df[["start", "end"]] = df[["start", "end"]].astype('float')
    df = df.round(0)
    df[["start", "end"]] = df[["start", "end"]].astype('int')

    df.label = df.label.str.strip().str.upper()

    # It will modify the limits of partially selected labels
    # Given end and start may be in the middle of a label
    if start:
        df = df[df.end > start]
        df.loc[df.start < start, "start"] = start
        df = df[df.start >= start]
    if end:
        df = df[df.start < end]
        df.loc[df.end > end, "end"] = end
        df = df[df.end <= end]
    # names_of_interest = _names_of_interest
    if verbose:
        print("Labels in (", start, ",", end, ") from", filename, "\n", df.label.unique())

    if activities_of_interest:
        activities_of_interest = [act.strip().upper() for act in activities_of_interest]
        df = df.loc[df.label.isin(activities_of_interest)]

    segments = [Segment(start, end, label) for name, (start, end, label) in df.iterrows()]
    indexes = [np.arange(start, end) for name, (start, end, label) in df.iterrows()]
    if len(segments) < 1:
        print("Warning, you are trying to load a span with no labels from:", filename)

    frames = segments2frames(segments)
    indexes = np.concatenate(indexes)
    serie = pd.Series(frames, index=indexes)

    if serie.index.has_duplicates:
        print("Overlapping labels were found in", filename)
        print("Check labels corresponding to times given below (in seconds):")
        print(serie.index[serie.index.duplicated()])

    s_formatted = serie.reindex(np.arange(serie.index[-1]), fill_value="")

    return s_formatted
